﻿function New-SignedScript	{
<#
.SYNOPSIS
	Digitally signs a PowerShell script using a code signing certificate.

.DESCRIPTION
	Digitally signs a PowerShell script using a code signing certificate installed on the local machine.

.NOTES
  Version							: 1.3 - See changelog at http://www.ehloworld.com/2723
	Wish list						:
  Rights Required			: None
  Sched Task Required	: No
  Lync Server Version	: 2013 (tested through CU3)
  Author/Copyright		: © Pat Richard, Skype for Business MVP - All Rights Reserved
  Email/Blog/Twitter	: pat@innervation.com   http://www.ehloworld.com    @patrichard
  Dedicated Post			: http://www.ehloworld.com/1496
  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is
  											provided AS IS without warranty of any kind. I disclaim all implied warranties including,
  											without limitation, any implied warranties of merchantability or of fitness for a particular
  											purpose. The entire risk arising out of the use or performance of the sample scripts and
  											documentation remains with you. In no event shall I be liable for any damages whatsoever
  											(including, without limitation, damages for loss of business profits, business interruption,
  											loss of business information, or other pecuniary loss) arising out of the use of or inability
  											to use the script or documentation.
  Acknowledgements 		:
  Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
  Limitations					:
  Known issues				: None yet, but I'm sure you'll find some!

.LINK
	http://www.ehloworld.com/1496

.EXAMPLE
	New-SignedScript .\MyScript.ps1

	Description
	-----------
	Runs script with default values. If there is a single code signing certificate, it will be used. If there is more than one valid code signing certificate, a popup will display the certificates and allow one to be chosen.

.EXAMPLE
	New-SignedScript .\MyScript.ps1 -CodeSigningCert 0

	Description
	-----------
	Signs the specified script, MyScript.ps1, with the first code signing certificate found.
	
.INPUTS
	None. You cannot pipe objects to this script.
#>
	#Requires -Version 3.0

	[CmdletBinding(SupportsShouldProcess = $True)]
	param (
		# Path (including file name) of script to sign
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true, HelpMessage = "No filename specified")]
		[ValidatePattern(".ps1$|.psm1$")]
		[ValidateNotNullOrEmpty()]
		[string] $FileName,
		
		# Index number of code signing certificate to use
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true)]
		[string] $CodeSigningCerificate
	)
	BEGIN	{
    Write-Verbose "Getting certificate available for codesigning"
    # [int] $totalvalidcerts = (Get-ChildItem Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.NotAfter -gt (Get-Date)}).length
    [int] $totalvalidcerts = (Get-ChildItem Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.Verify()}).length
    if ($totalvalidcerts -eq 1){
      # [Security.Cryptography.X509Certificates.X509Certificate2] $cert = (Get-ChildItem Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.NotAfter -ge (Get-Date)})[0]
      [Security.Cryptography.X509Certificates.X509Certificate2] $cert = (Get-ChildItem Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.Verify()})[0]
    }
    elseif (($CodeSigningCerificate) -and ($totalvalidcerts -gt 1)){
      # [Security.Cryptography.X509Certificates.X509Certificate2] $cert = (Get-ChildItem Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.NotAfter -ge (Get-Date)})[$CodeSigningCerificate]
      [Security.Cryptography.X509Certificates.X509Certificate2] $cert = (Get-ChildItem Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.Verify()})[$CodeSigningCerificate]
    }else{
      # [Security.Cryptography.X509Certificates.X509Certificate2] $cert = (Get-ChildItem Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.NotAfter -ge (Get-Date)} | Out-GridView -Title "Valid certificates" -OutputMode Single)[0]
      [Security.Cryptography.X509Certificates.X509Certificate2] $cert = (Get-ChildItem Cert:\CurrentUser\My -CodeSigningCert | Where-Object {$_.Verify()} | Out-GridView -Title "Valid certificates" -OutputMode Single)[0]
    }
	}
	PROCESS	{
    if ($cert){
	    Write-Verbose "$totalvalidcerts valid certificate(s) found"
	    Write-Verbose "Signing `"$FileName`" with $cert"
	    Set-AuthenticodeSignature -FilePath $FileName -Certificate $cert -TimestampServer "http://timestamp.digicert.com" | Out-Null
	    Write-Verbose "Checking validity"
      $valid = ((Get-AuthenticodeSignature $FileName).status -Match "Valid")
      if ($valid){
  	    Write-Verbose "Signing $FileName succeeded"
      }else{
  	    Write-Error "Signing $FileName failed"
      }
    }else{
	    Write-Error "No valid codesigning certificate found"
    }		
	}
	END	{
		Remove-Variable Valid
		Remove-Variable FileName
		Remove-Variable Cert
	}
} # end function New-SignedScript
